@@ -28,37 +28,37 @@
 
 #if (FREETYPE_MAJOR==2) && ((FREETYPE_MINOR<1) || ((FREETYPE_MINOR==1) && (FREETYPE_PATCH<8)))
 
-#include "ftfont-old.m"
+#import "ftfont-old.m"
 
 #else
 
 
 #include <math.h>
 
-#include <Foundation/NSObject.h>
-#include <Foundation/NSArray.h>
-#include <Foundation/NSSet.h>
-#include <Foundation/NSDictionary.h>
-#include <Foundation/NSValue.h>
-#include <Foundation/NSPathUtilities.h>
-#include <Foundation/NSFileManager.h>
-#include <Foundation/NSUserDefaults.h>
-#include <Foundation/NSBundle.h>
-#include <Foundation/NSDebug.h>
-#include <GNUstepBase/Unicode.h>
+#import <Foundation/NSObject.h>
+#import <Foundation/NSArray.h>
+#import <Foundation/NSCharacterSet.h>
+#import <Foundation/NSSet.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSValue.h>
+#import <Foundation/NSPathUtilities.h>
+#import <Foundation/NSFileManager.h>
+#import <Foundation/NSUserDefaults.h>
+#import <Foundation/NSBundle.h>
+#import <Foundation/NSDebug.h>
+#import <GNUstepBase/Unicode.h>
 
-#include <GNUstepGUI/GSFontInfo.h>
-#include <AppKit/NSAffineTransform.h>
-#include <AppKit/NSBezierPath.h>
+#import <GNUstepGUI/GSFontInfo.h>
+#import <AppKit/NSAffineTransform.h>
+#import <AppKit/NSBezierPath.h>
 
-//#include "gsc/GSContext.h"
-#include "gsc/GSGState.h"
+#import "gsc/GSGState.h"
 
-#include "ftfont.h"
-#include "FTFontEnumerator.h"
-#include "FTFaceInfo.h"
+#import "ftfont.h"
+#import "FTFontEnumerator.h"
+#import "FTFaceInfo.h"
 
-#include "blit.h"
+#import "blit.h"
 
 
 #define DI (*di)
@@ -113,7 +113,7 @@ static int subpixel_text;
   unsigned int cachedGlyph[CACHE_SIZE];
   NSSize cachedSize[CACHE_SIZE];
 
-  float lineHeight;
+  CGFloat lineHeight;
 }
 @end
 
@@ -167,7 +167,7 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib,
 @implementation FTFontInfo
 
 - (id) initWithFontName: (NSString *)name
-                 matrix: (const float *)fmatrix
+                 matrix: (const CGFloat *)fmatrix
              screenFont: (BOOL)p_screenFont
 {
   NSArray *rfi;
@@ -285,7 +285,7 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib,
     FT_Face face = ft_size->face;
     FT_CharMap cmap;
     FT_Encoding e;
-    unicodeCmap = 0;
+    unicodeCmap = -1;
 
     if (!face)
       {
@@ -411,7 +411,7 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib,
   return coveredCharacterSet;
 }
 
-- (float) defaultLineHeightForFont
+- (CGFloat) defaultLineHeightForFont
 {
   return lineHeight;
 }
@@ -1753,7 +1753,7 @@ static FT_Error ft_get_face(FTC_FaceID fid, FT_Library lib,
 }
 
 
-- (float) widthOfString: (NSString*)string
+- (CGFloat) widthOfString: (NSString*)string
 {
   unichar ch;
   int i, c = [string length];
