@@ -27,6 +27,7 @@
 #include "config.h"
 #import <Foundation/NSObjCRuntime.h>
 #import <Foundation/NSValue.h>
+#import <Foundation/NSDictionary.h>
 #import <AppKit/NSAffineTransform.h>
 #import <AppKit/NSBezierPath.h>
 #import <AppKit/NSColor.h>
@@ -1171,7 +1172,7 @@ typedef enum {
   rect = [function affectedRect];
   iwidth = rect.size.width;
   iheight = rect.size.height;
-  data = objc_malloc(sizeof(char) * iwidth * iheight * 4);
+  data = malloc(sizeof(char) * iwidth * iheight * 4);
   i = 0;
 
   for (y = NSMinY(rect); y < NSMaxY(rect); y++)
@@ -1217,7 +1218,7 @@ typedef enum {
         : 32 : 4 * iwidth  : NO
         : YES : NSDeviceRGBColorSpace
         : (const unsigned char **)&data];
-  objc_free(data);
+  free(data);
 
   DESTROY(matrix);
   DESTROY(inverse);
@@ -1254,7 +1255,8 @@ typedef enum {
 
   size = [pattern size];
   y = floor(NSMinY(rect) / size.height) * size.height;
-  while (y < NSMaxY(rect))
+
+    while (y < NSMaxY(rect))
     {
       x = floor(NSMinX(rect) / size.width) * size.width;
       while (x < NSMaxX(rect))
@@ -1277,12 +1279,14 @@ typedef enum {
   void *oldClip;
 
   oldClip = [self saveClip];
-  path = [NSBezierPath bezierPathWithRect: rect];
+  path = [[NSBezierPath alloc] init];
+  [path appendBezierPathWithRect: rect];
   [self DPSclip];
 
   [self _fillRect: rect withPattern: color_pattern];
 
   [self restoreClip: oldClip];
+  RELEASE(path);
   path = oldPath;
 }
 
