@@ -60,6 +60,8 @@
 #include <sys/file.h>
 #endif
 
+#include <math.h>
+
 static BOOL _enableCallbacks = YES;
 
 static NSEvent *process_key_event(WIN32Server *svr, 
@@ -195,8 +197,8 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
   wc.style = CS_HREDRAW | CS_VREDRAW; 
   wc.lpfnWndProc = (WNDPROC) MainWndProc; 
   wc.cbClsExtra = 0; 
-  // Keep extra space for each window, for GS data
-  wc.cbWndExtra = sizeof(WIN_INTERN); 
+  // Keep extra space for each window, for OFF_LEVEL and OFF_ORDERED
+  wc.cbWndExtra = WIN_EXTRABYTES; 
   wc.hInstance = hinstance; 
   wc.hIcon = NULL;//currentAppIcon;
   wc.hCursor = LoadCursor(NULL, IDC_ARROW);
@@ -457,7 +459,7 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
         wstyle |= WS_MINIMIZEBOX + WS_SYSMENU;
 
       if ((style & NSResizableWindowMask) == NSResizableWindowMask)
-        wstyle |= WS_SIZEBOX;
+        wstyle |= WS_SIZEBOX + WS_MAXIMIZEBOX;
 
       if (((style & NSMiniWindowMask) == NSMiniWindowMask)
           || ((style & NSIconWindowMask) == NSIconWindowMask))
@@ -475,17 +477,40 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
 {
   DWORD estyle = 0;
 
-  if ((style & NSMiniaturizableWindowMask) == NSMiniaturizableWindowMask)
+  if ((style & NSUtilityWindowMask) == NSUtilityWindowMask)
     {
-      if ([self usesNativeTaskbar])
-        estyle = WS_EX_APPWINDOW;
-      else
-        estyle = WS_EX_TOOLWINDOW;
+      // WS_EX_TOOLWINDOW gives windows a thinner frame, like NSUtilityWindowMask
+      estyle |= WS_EX_TOOLWINDOW;
     }
-  else
+
+  if ([self usesNativeTaskbar])
     {
-      estyle = WS_EX_TOOLWINDOW;
-    } 
+      // We will put all bordered windows except utility windows in the
+      // taskbar. Utility windows don't need to be in the taskbar since
+      // they are in the floating window level, so always visible.
+
+      if (style == NSBorderlessWindowMask)
+        {
+          // WS_EX_TOOLWINDOW also prevents windows from appearing in the taskbar.
+          estyle |= WS_EX_TOOLWINDOW;
+        }
+      else if ((style & NSUtilityWindowMask) == 0)
+        {
+          // WS_EX_APPWINDOW requests that the window appear in the taskbar
+          estyle |= WS_EX_APPWINDOW;
+        }
+   }
+  else /* (NO == [self usesNativeTaskbar]) */
+   {
+      // Prevent all windows from appearing in the taskbar. As an undesired 
+      // side effect this will give all windows with frames thin "tool window" 
+      // frames. We could also get rid of the taskbar buttons by creating
+      // a hidden window, and setting it as the parent of all other windows, 
+      // but that would be more complicated to manage.
+      // See http://msdn.microsoft.com/en-us/library/bb776822(v=VS.85).aspx#Managing_Taskbar_But
+ 
+      estyle |= WS_EX_TOOLWINDOW;
+    }
 
   return estyle;
 }
@@ -666,6 +691,9 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
       case WM_COMMAND: 
         [self decodeWM_COMMANDParams: wParam : lParam : hwnd];
         break;
+      case WM_THEMECHANGED: 
+        [self decodeWM_THEMECHANGEDParams: wParam : lParam : hwnd];
+        break;
       case WM_SYSKEYDOWN:  //KEYBOARD
         NSDebugLLog(@"NSEvent", @"Got Message %s for %d", "SYSKEYDOWN", hwnd);
         ev = process_key_event(self, hwnd, wParam, lParam, NSKeyDown);
@@ -897,8 +925,11 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
                         (HMENU)NULL, 
                         hinstance, 
                         (void*)type);
-  NSDebugLLog(@"WTrace", @"         num/handle: %d", hwnd);
-
+  NSDebugLLog(@"WCTrace", @"         num/handle: %d", hwnd);
+  if (!hwnd) {
+    NSLog(@"CreateWindowEx Failed %d", GetLastError());
+  }
+  
   SetLayeredWindowAttributes(hwnd, 0, 255, LWA_ALPHA);
 
   [self _setWindowOwnedByServer: (int)hwnd];
@@ -907,8 +938,10 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
 
 - (void) termwindow: (int) winNum
 {
-  NSDebugLLog(@"WTrace", @"termwindow: %d", winNum);
-  DestroyWindow((HWND)winNum); 
+  NSDebugLLog(@"WCTrace", @"termwindow: %d", winNum);
+  if (!DestroyWindow((HWND)winNum)) {
+    NSLog(@"DestroyWindow Failed %d", GetLastError());
+  }
 }
 
 - (void) stylewindow: (unsigned int)style : (int) winNum
@@ -1041,7 +1074,7 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
     {
       SetWindowLong((HWND)winNum, OFF_ORDERED, 0);
       SetWindowPos((HWND)winNum, NULL, 0, 0, 0, 0, 
-        SWP_HIDEWINDOW | SWP_NOSIZE | SWP_NOMOVE | SWP_NOZORDER);
+        SWP_HIDEWINDOW | SWP_NOSIZE | SWP_NOMOVE | SWP_NOZORDER | SWP_NOACTIVATE);
       return;
     }
 
@@ -1099,7 +1132,10 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
 
   if (level <= NSDesktopWindowLevel)
     {
+      HWND desktop = GetDesktopWindow();
+
       // For desktop level, put this at the bottom of the z-order
+      SetParent((HWND)winNum, desktop);
       otherWin = (int)HWND_BOTTOM;
       NSDebugLLog(@"WTrace",
 	@"orderwindow: set %i (%i) to bottom", winNum, level);
@@ -1305,13 +1341,11 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
 
 - (void) setwindowlevel: (int) level : (int) winNum
 {
-  WIN_INTERN *win = (WIN_INTERN *)GetWindowLongPtr((HWND)winNum, GWLP_USERDATA);
-
   NSDebugLLog(@"WTrace", @"setwindowlevel: %d : %d", level, winNum);
-  if (win->level != level)
+  if (GetWindowLong((HWND)winNum, OFF_LEVEL) != level)
     {
       SetWindowLong((HWND)winNum, OFF_LEVEL, level);
-      if (win->orderedIn == YES)
+      if (GetWindowLong((HWND)winNum, OFF_ORDERED) == YES)
 	{
           [self orderwindow: NSWindowAbove : 0 : winNum];
 	}
@@ -1320,8 +1354,7 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
 
 - (int) windowlevel: (int) winNum
 {
-  WIN_INTERN *win = (WIN_INTERN *)GetWindowLongPtr((HWND)winNum, GWLP_USERDATA);
-  return win->level;
+  return GetWindowLong((HWND)winNum, OFF_LEVEL);
 }
 
 - (NSArray *) windowlist
@@ -1375,6 +1408,18 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
   p.x = size.width;
   p.y = size.height;
   win->minmax.ptMaxTrackSize = p;
+
+  // Disable the maximize box if a maximum size is set
+  if (size.width < 10000 || size.height < 10000)
+    {
+      SetWindowLong((HWND)winNum, GWL_STYLE, 
+          GetWindowLong((HWND)winNum, GWL_STYLE) ^ WS_MAXIMIZEBOX);
+    }
+  else
+    {
+      SetWindowLong((HWND)winNum, GWL_STYLE, 
+          GetWindowLong((HWND)winNum, GWL_STYLE) | WS_MAXIMIZEBOX);
+    }
 }
 
 /** Set the minimum size of the window */
@@ -1505,9 +1550,15 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
   POINT p;
 
   if (!GetCursorPos(&p))
-    {  
-      NSLog(@"GetCursorPos failed with %d", GetLastError());
-      return NSZeroPoint;
+    { 
+	  // Try using cursorInfo which should work in more situations
+	  CURSORINFO cursorInfo;
+	  cursorInfo.cbSize = sizeof(CURSORINFO); 
+	  if (!GetCursorInfo(&cursorInfo)) {
+		NSLog(@"GetCursorInfo failed with %d", GetLastError());
+        return NSZeroPoint;
+      }
+	  p = cursorInfo.ptScreenPos;
     }
 
   return MSScreenPointToGS(p.x, p.y);
@@ -1573,13 +1624,13 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
   *cid = (void*)hCursor;
 }
 
-- (void) imagecursor: (NSPoint)hotp : (int) w :  (int) h 
-		    : (int)colors : (const unsigned char *)image : (void **)cid
+- (void) imagecursor: (NSPoint)hotp : (NSImage *)image : (void **)cid
 {
   /*
     HCURSOR cur;
     BYTE *and;
     BYTE *xor;
+    int w, h;
 
     xor = image;
     cur = CreateCursor(hinstance, (int)hotp.x, (int)hotp.y,  (int)w, (int)h, and, xor);
@@ -1587,7 +1638,7 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
     */
 }
 
-- (void) setcursorcolor: (NSColor *)fg : (NSColor *)bg : (void*) cid
+- (void) recolorcursor: (NSColor *)fg : (NSColor *)bg : (void*) cid
 {
   /* FIXME The colour is currently ignored
      if (fg != nil)
@@ -1600,10 +1651,19 @@ LRESULT CALLBACK MainWndProc(HWND hwnd, UINT uMsg,
      }
      }
   */
+}
 
+- (void) setcursor: (void*) cid
+{
   SetCursor((HCURSOR)cid);
 }
 
+- (void) freecursor: (void*) cid
+{
+  // This is only allowed on non-shared cursors and we have no way of knowing that.
+  //DestroyCursor((HCURSOR)cid);
+}
+
 - (void) setParentWindow: (int)parentWin 
           forChildWindow: (int)childWin
 {
@@ -1620,7 +1680,7 @@ process_char(WPARAM wParam, unsigned *eventModifierFlags)
       case VK_RETURN: return NSCarriageReturnCharacter;
       case VK_TAB:    return NSTabCharacter;
       case VK_ESCAPE:  return 0x1b;
-      case VK_BACK:   return NSBackspaceCharacter;
+      case VK_BACK:   return NSDeleteCharacter;
 
 	/* The following keys need to be reported as function keys */
   #define WIN_FUNCTIONKEY \
@@ -1656,20 +1716,21 @@ process_char(WPARAM wParam, unsigned *eventModifierFlags)
       case VK_RIGHT:       WIN_FUNCTIONKEY return NSRightArrowFunctionKey;
       case VK_UP:          WIN_FUNCTIONKEY return NSUpArrowFunctionKey;
       case VK_DOWN:        WIN_FUNCTIONKEY return NSDownArrowFunctionKey;
-      case VK_PRIOR:       WIN_FUNCTIONKEY return NSPrevFunctionKey;
-      case VK_NEXT:        WIN_FUNCTIONKEY return NSNextFunctionKey;
+      case VK_PRIOR:       WIN_FUNCTIONKEY return NSPageUpFunctionKey;
+      case VK_NEXT:        WIN_FUNCTIONKEY return NSPageDownFunctionKey;
       case VK_END:         WIN_FUNCTIONKEY return NSEndFunctionKey;
-	//case VK_BEGIN:       WIN_FUNCTIONKEY return NSBeginFunctionKey;
       case VK_SELECT:      WIN_FUNCTIONKEY return NSSelectFunctionKey;
       case VK_PRINT:       WIN_FUNCTIONKEY return NSPrintFunctionKey;
+      case VK_SNAPSHOT:    WIN_FUNCTIONKEY return NSPrintScreenFunctionKey;
       case VK_EXECUTE:     WIN_FUNCTIONKEY return NSExecuteFunctionKey;
       case VK_INSERT:      WIN_FUNCTIONKEY return NSInsertFunctionKey;
       case VK_HELP:        WIN_FUNCTIONKEY return NSHelpFunctionKey;
       case VK_CANCEL:      WIN_FUNCTIONKEY return NSBreakFunctionKey;
-	//case VK_MODECHANGE:  WIN_FUNCTIONKEY return NSModeSwitchFunctionKey;
+      case VK_MODECHANGE:  WIN_FUNCTIONKEY return NSModeSwitchFunctionKey;
       case VK_SCROLL:      WIN_FUNCTIONKEY return NSScrollLockFunctionKey;
       case VK_PAUSE:       WIN_FUNCTIONKEY return NSPauseFunctionKey;
       case VK_OEM_CLEAR:   WIN_FUNCTIONKEY return NSClearDisplayFunctionKey;
+      case VK_CLEAR:       WIN_FUNCTIONKEY return NSClearLineFunctionKey;  
   #undef WIN_FUNCTIONKEY
       default: 
 	return 0;
@@ -1764,7 +1825,7 @@ process_key_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam,
     = MSWindowPointToGS(svr, hwnd,  GET_X_LPARAM(pos), GET_Y_LPARAM(pos));
 
   ltime = GetMessageTime();
-  time = ltime / 1000;
+  time = ltime / 1000.0f;
 
   GetKeyboardState(keyState);
   eventFlags = mask_for_keystate(keyState);
@@ -1904,7 +1965,7 @@ process_mouse_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam,
 
   eventLocation = MSWindowPointToGS(svr, hwnd, clientX, clientY);
   ltime = GetMessageTime();
-  time = ltime / 1000;
+  time = ltime / 1000.0f;
   tick = GetTickCount();
   eventFlags = 0;
   if (wParam & MK_CONTROL)
@@ -2018,15 +2079,21 @@ process_mouse_event(WIN32Server *svr, HWND hwnd, WPARAM wParam, LPARAM lParam,
       if (ltime == lastTime) // duplicate event has identical time
 	return nil; // ignore it
 
-      if (lastTime + GetDoubleClickTime() > ltime)
+      static NSPoint lastClick = {0.0, 0.0};
+      
+      if (lastTime + GetDoubleClickTime() > ltime 
+          && fabs(eventLocation.x - lastClick.x) < GetSystemMetrics(SM_CXDOUBLECLK)
+          && fabs(eventLocation.y - lastClick.y) < GetSystemMetrics(SM_CYDOUBLECLK))
         {
           clickCount += 1;
         }
       else 
         {
           clickCount = 1;
-          lastTime = ltime;
         }
+      lastTime = ltime;  
+      lastClick = eventLocation;
+        
       SetCapture(hwnd); // capture the mouse to get mouse moved events outside of window
     }
   else if ( ((eventType == NSLeftMouseUp)
