@@ -39,16 +39,20 @@
 #endif
 #endif
 
-#include <AppKit/NSAffineTransform.h>
-#include <AppKit/NSBezierPath.h>
-#include <AppKit/NSColor.h>
-#include <AppKit/NSFont.h>
-#include <AppKit/NSGraphics.h>
+#import <Foundation/NSData.h>
+#import <Foundation/NSDictionary.h>
+#import <Foundation/NSString.h>
 
-#include "winlib/WIN32GState.h"
-#include "winlib/WIN32Context.h"
-#include "winlib/WIN32FontInfo.h"
-#include "win32/WIN32Server.h"
+#import <AppKit/NSAffineTransform.h>
+#import <AppKit/NSBezierPath.h>
+#import <AppKit/NSColor.h>
+#import <AppKit/NSFont.h>
+#import <AppKit/NSGraphics.h>
+
+#import "winlib/WIN32GState.h"
+#import "winlib/WIN32Context.h"
+#import "winlib/WIN32FontInfo.h"
+#import "win32/WIN32Server.h"
 
 #include <math.h>
 #include <limits.h>
@@ -246,6 +250,54 @@ BOOL alpha_blend_source_over(HDC destDC,
     wscolor = RGB(color.field[0]*255, color.field[1]*255, color.field[2]*255);
 }
 
+// NOTE: Ideally this code should be moved to -DPSinitgraphics and not called 
+// directly, but this would imply to rewrite WIN32GState more extensively since 
+// the current code expects the GDI base coordinates in many methods.
+- (void) setUpAppKitBaseCoordinatesForHDC: (HDC)hDC
+{
+  int surfaceHeight = WindowHeight(window);
+  int bottomOffset = offset.y - surfaceHeight;
+  XFORM xForm;
+
+  // NOTE: The world transform is the GDI CTM.
+  GetWorldTransform(hDC, &oldWorldTransform);
+
+  // Enables the use of transforms
+  SetGraphicsMode(hDC, GM_ADVANCED);
+  // Maps units to pixels
+  SetMapMode(hDC, MM_TEXT);
+
+  /* Flip the GDI base coordinate to match the AppKit because GDI draws the 
+     other way around with MM_TEXT.
+     Also offset the drawing area to take in account the window border. */
+
+  xForm.eM11 = (FLOAT)1; 
+  xForm.eM12 = (FLOAT)0; 
+  xForm.eM21 = (FLOAT)0; 
+  xForm.eM22 = (FLOAT)-1; 
+  xForm.eDx  = (FLOAT)-offset.x; 
+  xForm.eDy  = (FLOAT)surfaceHeight + bottomOffset;
+
+  SetWorldTransform(hDC, &xForm);
+}
+
+- (void) restoreGDIBaseCoordinatesForHDC: (HDC)hDC
+{
+  SetWorldTransform(hDC, &oldWorldTransform);
+  SetGraphicsMode(hDC, GM_COMPATIBLE);
+}
+
+/* For debugging */
+- (void) drawOrientationMarkersIn: (HDC)hDC
+{
+  RECT rect1 = { 0, 0, 20, 10 };
+  RECT rect2 = { 0, 30, 20, 30 + 10 };
+  FillRect(hDC, &rect1, CreateSolidBrush(RGB(255, 0, 255)));
+  FillRect(hDC, &rect2, CreateSolidBrush(RGB(0, 255, 0)));
+}
+
+// FIXME: Drawing rotated images is broken (the origin is wrong).
+// If rewritten, viewIsFlipped  check must be removed.
 - (void) compositeGState: (WIN32GState *)source
 		fromRect: (NSRect)sourceRect
 		 toPoint: (NSPoint)destPoint
@@ -356,7 +408,6 @@ BOOL alpha_blend_source_over(HDC destDC,
                     op: (NSCompositingOperation)op
 {
   float gray;
-  BOOL success = NO;
 
   // FIXME: This is taken from the xlib backend
   [self DPScurrentgray: &gray];
@@ -404,6 +455,115 @@ BOOL alpha_blend_source_over(HDC destDC,
     }
 }
 
+// FIXME: Drawing images with alpha blending is broken
+- (void) drawGState: (WIN32GState *)source 
+           fromRect: (NSRect)aRect 
+            toPoint: (NSPoint)aPoint 
+                 op: (NSCompositingOperation)op
+           fraction: (float)delta
+{
+  HDC sourceDC;
+  HDC hDC;
+  XFORM xForm, xForm2;
+  NSAffineTransformStruct tstruct = [ctm transformStruct];
+  RECT rectFrom = GSWindowRectToMS(source, aRect);
+  int x = aPoint.x;
+  int y = aPoint.y;
+  int w = aRect.size.width;
+  int h = aRect.size.height;
+  BOOL success = YES;
+
+  sourceDC = [source getHDC];
+  if (!sourceDC)
+    {
+      return;
+    } 
+
+  if (self == source)
+    {
+      hDC = sourceDC;
+    }
+  else
+    {
+      hDC = [self getHDC];
+      if (!hDC)
+        {
+          [source releaseHDC: sourceDC];
+          return;
+        }
+    }
+
+  /* Set up the AppKit base coordinates as the World transform */
+
+  [self setUpAppKitBaseCoordinatesForHDC: hDC];
+
+  /* Apply the AppKit CTM */
+
+  xForm.eM11 = (FLOAT)tstruct.m11; 
+  xForm.eM12 = (FLOAT)tstruct.m12; 
+  xForm.eM21 = (FLOAT)tstruct.m21; 
+  xForm.eM22 = (FLOAT)tstruct.m22; 
+  xForm.eDx  = (FLOAT)tstruct.tX;
+  xForm.eDy  = (FLOAT)tstruct.tY;
+
+  // Concat the transform by prepending it to the CTM
+  ModifyWorldTransform(hDC, &xForm, MWT_LEFTMULTIPLY);
+
+  /* Flip the CTM to compensate the fact that images are drawn upside down 
+     by -DPSimage which uses the GDI top left origin coordinates. */
+
+  xForm2.eM11 = (FLOAT)1;
+  xForm2.eM12 = (FLOAT)0; 
+  xForm2.eM21 = (FLOAT)0; 
+  xForm2.eM22 = (FLOAT)-1; 
+  xForm2.eDx  = (FLOAT)0; 
+  xForm2.eDy  = (FLOAT)y * 2 + aRect.size.height;
+
+  // Concat the flip transform by prepending it to the CTM
+  ModifyWorldTransform(hDC, &xForm2, MWT_LEFTMULTIPLY);
+  
+  switch (op)
+    {
+    case NSCompositeSourceOver:
+      {
+	success = alpha_blend_source_over(hDC, 
+					  sourceDC, 
+					  rectFrom, 
+					  x, y, w, h, 
+					  delta);
+	if (success)
+	  break;
+      }
+    case NSCompositeCopy:
+      {
+        success = BitBlt(hDC, x, y, w, h,
+                         sourceDC, rectFrom.left, rectFrom.top, SRCCOPY);
+        break;	      
+      }
+    case NSCompositeClear:
+      {
+        break;
+      }
+    default:
+      success = BitBlt(hDC, x, y, w, h,
+                       sourceDC, rectFrom.left, rectFrom.top, SRCCOPY);
+      break;
+    }
+  //[self drawOrientationMarkersIn: hDC];
+
+  if (!success)
+    {
+      NSLog(@"Blit operation failed %d", GetLastError());
+    }
+
+  [self restoreGDIBaseCoordinatesForHDC: hDC];
+
+  if (self != source)
+    {
+      [self releaseHDC: hDC];
+    }
+  [source releaseHDC: sourceDC];
+}
 
 static
 HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
@@ -449,28 +609,39 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
 
   if (!(GetDeviceCaps(hDC, RASTERCAPS) &  RC_DI_BITMAP)) 
     {
+      NSLog(@"Device %d does not support bitmap operations", hDC);
       return NULL;
     }
 
   hbitmap = CreateCompatibleBitmap(hDC, pixelsWide, pixelsHigh);
   if (!hbitmap)
     {
+      NSLog(@"Failed to CreateCompatibleBitmap (%d, %d). Error %d", 
+            pixelsWide, pixelsHigh, GetLastError());
       return NULL;
     }
 
   if (bitsPerPixel > 8)
     {
-      bitmap = objc_malloc(sizeof(BITMAPV4HEADER));
+      bitmap = malloc(sizeof(BITMAPV4HEADER));
     }
   else 
     {
-      bitmap = objc_malloc(sizeof(BITMAPINFOHEADER) +  
+      // Leave some extra space for colour map. (Currently not used)
+      bitmap = malloc(sizeof(BITMAPINFOHEADER) +  
 			   (1 << bitsPerPixel) * sizeof(RGBQUAD));
     }
-  bmih = (BITMAPINFOHEADER*)bitmap;
+  if (!bitmap)
+    {
+       NSLog(@"Failed to allocate memory for bitmap. Error %d", GetLastError());
+       DeleteObject(hbitmap);
+       return NULL;
+    }
 
+  bmih = (BITMAPINFOHEADER*)bitmap;
   bmih->biSize = sizeof(BITMAPINFOHEADER);
   bmih->biWidth = pixelsWide;
+  // Top down orientation
   bmih->biHeight = -pixelsHigh;
   bmih->biPlanes = 1;
   bmih->biBitCount = bitsPerPixel;
@@ -484,14 +655,7 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
   bmih->biClrImportant = 0;
   fuColorUse = 0;
 
-  if (bitsPerPixel <= 8 && samplesPerPixel > 1)
-    {
-      // FIXME How to get a colour palette?
-      NSLog(@"Need to define colour map for images with %d bits", bitsPerPixel);
-      //bitmap->bmiColors;
-      fuColorUse = DIB_RGB_COLORS;
-    }
-  else if (bitsPerPixel == 8)
+  if (bitsPerPixel == 8 && samplesPerPixel == 1)
     {
       unsigned char* tmp;
       unsigned int pixels = pixelsHigh * pixelsWide;
@@ -503,8 +667,16 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
       NSDebugLLog(@"WIN32GState", @"8bit greyscale picture with pixelsWide:%d "
 	@"pixelsHigh:%d", pixelsWide, pixelsHigh);
       
-      tmp = objc_malloc(pixels * 4);
-      
+      tmp = malloc(pixels * 4);
+      if (!tmp)
+        {
+          NSLog(@"Failed to allocate temporary memory for bitmap. Error %d", 
+                GetLastError());
+          free(bitmap);
+          DeleteObject(hbitmap);
+          return NULL;
+        }
+
       if ([colorSpaceName isEqualToString: NSCalibratedWhiteColorSpace])
         {
           while (i < (pixels*4))
@@ -534,7 +706,13 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
             }
 	  }
       else
-      	NSLog(@"Unexpected condition, greyscale which is neither whte nor black calibrated");
+        {
+          NSLog(@"Unexpected condition, greyscale which is neither white nor black calibrated");
+          free(tmp);
+          free(bitmap);
+          DeleteObject(hbitmap);
+          return NULL;
+        }
       bits = tmp;
     }
   else if (bitsPerPixel == 32)
@@ -551,7 +729,16 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
       bmih->bV4GreenMask = 0x0000FF00;
       bmih->bV4RedMask = 0x00FF0000;
       bmih->bV4AlphaMask = 0xFF000000;
-      tmp = objc_malloc(pixels * 4);
+      tmp = malloc(pixels * 4);
+      if (!tmp)
+        {
+          NSLog(@"Failed to allocate temporary memory for bitmap. Error %d", 
+                GetLastError());
+          free(bitmap);
+          DeleteObject(hbitmap);
+          return NULL;
+        }
+
       while (i < pixels*4)
 	{
 	  tmp[i+0] = bits[i+2];
@@ -573,8 +760,16 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
       NSDebugLLog(@"WIN32GState", @"24bit picure with pixelsWide:%d "
 	@"pixelsHigh:%d", pixelsWide, pixelsHigh);
       
-      tmp = objc_malloc(pixels * 4);
-      //memset(tmp, 0xFF, pixels*4);
+      tmp = malloc(pixels * 4);
+      if (!tmp)
+        {
+          NSLog(@"Failed to allocate temporary memory for bitmap. Error %d", 
+                GetLastError());
+          free(bitmap);
+          DeleteObject(hbitmap);
+          return NULL;
+        }
+
       while (i < (pixels*4))
         {
           // We expand the bytes in a 24bit image into 32bits as Windows
@@ -589,24 +784,27 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
         }
       bits = tmp;
     }
-  else if (bitsPerPixel == 16)
+  else
     {
-/*
-      BITMAPV4HEADER *bmih;
-
-      bmih = (BITMAPV4HEADER*)bitmap;
-      bmih->bV4Size = sizeof(BITMAPV4HEADER);
-      bmih->bV4V4Compression = BI_BITFIELDS;
-      bmih->bV4RedMask = 0xF000;
-      bmih->bV4GreenMask = 0x0F00;
-      bmih->bV4BlueMask = 0x00F0;
-      bmih->bV4AlphaMask = 0x000F;
-*/
-      NSLog(@"Unsure how to handle images with %d bits", bitsPerPixel);
+      if (bitsPerPixel <= 8 && samplesPerPixel > 1)
+        {
+          // FIXME How to get a colour palette?
+          NSLog(@"Need to define colour map for images with %d bits", bitsPerPixel);
+          //bitmap->bmiColors;
+          //fuColorUse = DIB_RGB_COLORS;
+        }
+      else
+        {
+          NSLog(@"Unsure how to handle images with %d bits", bitsPerPixel);
+        }
+      free(bitmap);
+      DeleteObject(hbitmap);
+      return NULL;
     }
 
   if (!SetDIBits(hDC, hbitmap, 0, pixelsHigh, bits, bitmap, fuColorUse))
     {
+      NSLog(@"SetDIBits failed. Error %d", GetLastError());
       DeleteObject(hbitmap);
       hbitmap = NULL;
     }
@@ -614,12 +812,21 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
   if (bits != data[0])
     {
       /* cast bits to Void Pointer to fix warning in compile */
-      objc_free((void *)(bits));
+      free((void *)(bits));
     }
-  objc_free(bitmap);
+  free(bitmap);
   return hbitmap;
 }
 
+// NOTE: Draws the image with the GDI top left coordinate origin rather than 
+// the AppKit bottom left coordinate origin. Hence drawing from an image cache 
+// window into an AppKit base coordinate space results in an upside down image.
+// Ideally we should draw image in their cache windows with the AppKit base 
+// coordinates to prevent extra flip transforms to be required later on (see 
+// drawGState:fromRect:toPoint:op:fraction).
+// For some unknown reason, invoking -setUpAppKitBaseCoordinatesForHDC: in 
+// -DPSimage: can cause images to be drawn with a 1px horizontal line cut at 
+// the top. That's why -DPSimage we still use the GDI top left coordinates.
 - (void)DPSimage: (NSAffineTransform*) matrix 
 		: (int) pixelsWide : (int) pixelsHigh
 		: (int) bitsPerSample : (int) samplesPerPixel 
@@ -657,6 +864,7 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
     {
       NSLog(@"No DC for window %d in DPSImage. Error %d", 
 	    (int)window, GetLastError());
+      return;
     }
 
   hbitmap = GSCreateBitmap(hDC, pixelsWide, pixelsHigh,
@@ -667,6 +875,8 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
   if (!hbitmap)
     {
       NSLog(@"Created bitmap failed %d", GetLastError());
+      ReleaseDC((HWND)window, hDC);
+      return;
     }
 
   hDC2 = CreateCompatibleDC(hDC); 
@@ -674,12 +884,20 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
     {
       NSLog(@"No Compatible DC for window %d in DPSImage. Error %d", 
 	    (int)window, GetLastError());
+      DeleteObject(hbitmap);
+      ReleaseDC((HWND)window, hDC);
+      return;
     }
+
   old = SelectObject(hDC2, hbitmap);
   if (!old)
     {
       NSLog(@"SelectObject failed for window %d in DPSImage. Error %d", 
 	    (int)window, GetLastError());
+      DeleteDC(hDC2);
+      DeleteObject(hbitmap);
+      ReleaseDC((HWND)window, hDC);
+      return;
     }
 
   //SetMapMode(hDC2, GetMapMode(hDC));
@@ -698,12 +916,12 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
   pa[1] = GSViewPointToWin(self, NSMakePoint(pixelsWide, pixelsHigh));
   pa[2] = GSViewPointToWin(self, NSMakePoint(0, 0));
 
-  if (viewIsFlipped)
+  /*if (viewIsFlipped)
     {
       pa[0].y += pixelsHigh;
       pa[1].y += pixelsHigh;
       pa[2].y += pixelsHigh;
-    }
+    }*/
 
   if (old_ctm != nil)
     {
@@ -1087,14 +1305,14 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
       // The user has defined a dash pattern for stroking on
       // the path. Note that we lose the floating point information
       // here, as windows only supports DWORD elements, not float.
-      thePattern = objc_malloc(sizeof(float) * patternCount);
+      thePattern = malloc(sizeof(float) * patternCount);
       [path getLineDash: thePattern count: &patternCount phase: &phase];
 
-      iPattern = objc_malloc(sizeof(DWORD) * patternCount);
+      iPattern = malloc(sizeof(DWORD) * patternCount);
       int i  = 0;
       for (i = 0 ; i < patternCount; i ++)
 	iPattern[i] = (DWORD)thePattern[i];
-      objc_free(thePattern);
+      free(thePattern);
       thePattern = NULL;
     }
   else
@@ -1109,7 +1327,7 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
 
   if (iPattern)
     {
-      objc_free(iPattern);
+      free(iPattern);
       iPattern = NULL;
     }
 
@@ -1362,7 +1580,7 @@ HBITMAP GSCreateBitmap(HDC hDC, int pixelsWide, int pixelsHigh,
   // Gets the "bits" from the bitmap and copies them into a buffer 
   // which is pointed to by lpbi
   if (GetDIBits(hdcMemDC, hbitmap, 0, (UINT)bmpCopied.bmHeight, bits,
-    (BITMAPINFOHEADER *)lpbi, DIB_RGB_COLORS) == 0)
+    (LPBITMAPINFO)lpbi, DIB_RGB_COLORS) == 0)
     {
       NSLog(@"GetDIBits failed for window %d in GSReadRect. Error %d", 
 	(int)window, GetLastError());
