@@ -71,6 +71,7 @@
 // NumLock's mask (it depends on the keyboard mapping)
 static unsigned int _num_lock_mask;
 // Modifier state
+static char _shift_pressed = 0;
 static char _control_pressed = 0;
 static char _command_pressed = 0;
 static char _alt_pressed = 0;
@@ -190,22 +191,22 @@ static int check_modifier (XEvent *xEvent, KeySym key_sym)
     {
       NSLog(@"X-Windows error - %s\n\
           on display: %s\n\
-                    type: %d\n\
-       serial number: %d\n\
+                type: %d\n\
+       serial number: %lu\n\
         request code: %d\n",
         buffer,
-            XDisplayName(DisplayString(display)),
+        XDisplayName(DisplayString(display)),
         err->type, err->serial, err->request_code);
       return 0;
     }
   [NSException raise: NSWindowServerCommunicationException
     format: @"X-Windows error - %s\n\
           on display: %s\n\
-                    type: %d\n\
-       serial number: %d\n\
+                type: %d\n\
+       serial number: %lu\n\
         request code: %d\n",
         buffer,
-            XDisplayName(DisplayString(display)),
+        XDisplayName(DisplayString(display)),
         err->type, err->serial, err->request_code];
   return 0;
 }
@@ -323,7 +324,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
   o.x = x.x + l;
   o.y = NSHeight(win->xframe) - x.y + b;
 
-  NSDebugLLog(@"Frame", @"X2OP %d, %x, %@, %@", win->number, style,
+  NSDebugLLog(@"Frame", @"X2OP %lu, %x, %@, %@", win->number, style,
     NSStringFromPoint(x), NSStringFromPoint(o));
   return o;
 }
@@ -351,8 +352,8 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
     {
         // mouse button events
       case ButtonPress:
-        NSDebugLLog(@"NSEvent", @"%d ButtonPress: \
-                xEvent.xbutton.time %u timeOfLastClick %u \n",
+        NSDebugLLog(@"NSEvent", @"%lu ButtonPress: \
+                xEvent.xbutton.time %lu timeOfLastClick %lu \n",
                     xEvent.xbutton.window, xEvent.xbutton.time,
                     generic.lastClick);
         /*
@@ -505,7 +506,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
         break;
 
       case ButtonRelease:
-        NSDebugLLog(@"NSEvent", @"%d ButtonRelease\n",
+        NSDebugLLog(@"NSEvent", @"%lu ButtonRelease\n",
                     xEvent.xbutton.window);
         [self setLastTime: xEvent.xbutton.time];
         if (xEvent.xbutton.button == generic.lMouse)
@@ -559,12 +560,12 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
         break;
 
       case CirculateNotify:
-        NSDebugLLog(@"NSEvent", @"%d CirculateNotify\n",
+        NSDebugLLog(@"NSEvent", @"%lu CirculateNotify\n",
                     xEvent.xcirculate.window);
         break;
 
       case CirculateRequest:
-        NSDebugLLog(@"NSEvent", @"%d CirculateRequest\n",
+        NSDebugLLog(@"NSEvent", @"%lu CirculateRequest\n",
                     xEvent.xcirculaterequest.window);
         break;
 
@@ -573,7 +574,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
           NSTimeInterval time;
           DndClass dnd = xdnd ();
               
-          NSDebugLLog(@"NSEvent", @"%d ClientMessage - %s\n",
+          NSDebugLLog(@"NSEvent", @"%lu ClientMessage - %s\n",
 	    xEvent.xclient.window,
 	    XGetAtomName(dpy, xEvent.xclient.message_type));
           if (cWin == 0 || xEvent.xclient.window != cWin->ident)
@@ -608,7 +609,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
               else if ((Atom)xEvent.xclient.data.l[0]
                 == generic.miniaturize_atom)
                 {
-		  NSDebugLLog(@"Miniaturize", @"%d miniaturized", cWin->number);
+		  NSDebugLLog(@"Miniaturize", @"%lu miniaturized", cWin->number);
                   eventLocation = NSMakePoint(0,0);
                   e = [NSEvent otherEventWithType: NSAppKitDefined
                                location: eventLocation
@@ -796,14 +797,14 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
 
       case ColormapNotify:
         // colormap attribute
-        NSDebugLLog(@"NSEvent", @"%d ColormapNotify\n",
+        NSDebugLLog(@"NSEvent", @"%lu ColormapNotify\n",
                     xEvent.xcolormap.window);
         break;
 
             // the window has been resized, change the width and height
             // and update the window so the changes get displayed
       case ConfigureNotify:
-        NSDebugLLog(@"NSEvent", @"%d ConfigureNotify "
+        NSDebugLLog(@"NSEvent", @"%lu ConfigureNotify "
                     @"x:%d y:%d w:%d h:%d b:%d %c", xEvent.xconfigure.window,
                     xEvent.xconfigure.x, xEvent.xconfigure.y,
                     xEvent.xconfigure.width, xEvent.xconfigure.height,
@@ -853,7 +854,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
             cWin->xframe = x;
             n = [self _XFrameToOSFrame: x for: cWin];
             NSDebugLLog(@"Moving",
-                        @"Update win %d:\n   original:%@\n   new:%@",
+                        @"Update win %lu:\n   original:%@\n   new:%@",
                         cWin->number, NSStringFromRect(r), 
                         NSStringFromRect(x));
             /*
@@ -916,31 +917,31 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
             // same as ConfigureNotify but we get this event
             // before the change has actually occurred
       case ConfigureRequest:
-        NSDebugLLog(@"NSEvent", @"%d ConfigureRequest\n",
+        NSDebugLLog(@"NSEvent", @"%lu ConfigureRequest\n",
                     xEvent.xconfigurerequest.window);
         break;
 
             // a window has been created
       case CreateNotify:
-        NSDebugLLog(@"NSEvent", @"%d CreateNotify\n",
+        NSDebugLLog(@"NSEvent", @"%lu CreateNotify\n",
                     xEvent.xcreatewindow.window);
         break;
 
             // a window has been destroyed
       case DestroyNotify:
-        NSDebugLLog(@"NSEvent", @"%d DestroyNotify\n",
+        NSDebugLLog(@"NSEvent", @"%lu DestroyNotify\n",
                     xEvent.xdestroywindow.window);
         break;
 
             // when the pointer enters a window
       case EnterNotify:
-        NSDebugLLog(@"NSEvent", @"%d EnterNotify\n",
+        NSDebugLLog(@"NSEvent", @"%lu EnterNotify\n",
                     xEvent.xcrossing.window);
         break;
               
             // when the pointer leaves a window
       case LeaveNotify:
-        NSDebugLLog(@"NSEvent", @"%d LeaveNotify\n",
+        NSDebugLLog(@"NSEvent", @"%lu LeaveNotify\n",
                     xEvent.xcrossing.window);
         if (cWin == 0 || xEvent.xcrossing.window != cWin->ident)
           {
@@ -963,7 +964,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
 
             // the visibility of a window has changed
       case VisibilityNotify:
-        NSDebugLLog(@"NSEvent", @"%d VisibilityNotify %d\n", 
+        NSDebugLLog(@"NSEvent", @"%lu VisibilityNotify %d\n", 
                     xEvent.xvisibility.window, xEvent.xvisibility.state);
         if (cWin == 0 || xEvent.xvisibility.window != cWin->ident)
           {
@@ -994,7 +995,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
         // a portion of the window has become visible and
         // we must redisplay it
       case Expose:
-        NSDebugLLog(@"NSEvent", @"%d Expose\n",
+        NSDebugLLog(@"NSEvent", @"%lu Expose\n",
                     xEvent.xexpose.window);
         {
           BOOL isSubWindow = NO;
@@ -1074,7 +1075,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
 
       // keyboard focus entered a window
       case FocusIn:
-        NSDebugLLog(@"NSEvent", @"%d FocusIn\n",
+        NSDebugLLog(@"NSEvent", @"%lu FocusIn\n",
                     xEvent.xfocus.window);
         if (cWin == 0 || xEvent.xfocus.window != cWin->ident)
           {
@@ -1083,7 +1084,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
           }
         if (cWin == 0)
           break;
-        NSDebugLLog(@"Focus", @"%d got focus on %d\n",
+        NSDebugLLog(@"Focus", @"%lu got focus on %lu\n",
                     xEvent.xfocus.window, cWin->number);
         // Store this for debugging, may not be the real focus window
         generic.currentFocusWindow = cWin->number;
@@ -1112,7 +1113,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
            * If it has gone to a window - we see if it is one of ours.
            */
           XGetInputFocus(xEvent.xfocus.display, &fw, &rev);
-          NSDebugLLog(@"NSEvent", @"%d FocusOut\n",
+          NSDebugLLog(@"NSEvent", @"%lu FocusOut\n",
                       xEvent.xfocus.window);
           if (fw != None && fw != PointerRoot)
             {
@@ -1134,7 +1135,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
             {
               nswin = nil;
             }
-          NSDebugLLog(@"Focus", @"Focus went to %d (xwin %d)\n", 
+          NSDebugLLog(@"Focus", @"Focus went to %lu (xwin %lu)\n", 
                       (nswin != nil) ? cWin->number : 0, fw);
 
           // Focus went to a window not in this application.
@@ -1146,7 +1147,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
           // Clean up old focus request
           generic.cachedWindow
               = [XGServer _windowForXWindow: xEvent.xfocus.window];
-          NSDebugLLog(@"Focus", @"%d lost focus on %d\n",
+          NSDebugLLog(@"Focus", @"%lu lost focus on %lu\n",
                       xEvent.xfocus.window, (cWin) ? cWin->number : 0);
           generic.currentFocusWindow = 0;
           if (cWin && generic.desiredFocusWindow == cWin->number)
@@ -1158,7 +1159,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
         break;
 
       case GraphicsExpose:
-        NSDebugLLog(@"NSEvent", @"%d GraphicsExpose\n",
+        NSDebugLLog(@"NSEvent", @"%lu GraphicsExpose\n",
                     xEvent.xexpose.window);
         break;
 
@@ -1168,13 +1169,13 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
 
       // window is moved because of a change in the size of its parent
       case GravityNotify:
-        NSDebugLLog(@"NSEvent", @"%d GravityNotify\n",
+        NSDebugLLog(@"NSEvent", @"%lu GravityNotify\n",
                     xEvent.xgravity.window);
         break;
 
             // a key has been pressed
       case KeyPress:
-        NSDebugLLog(@"NSEvent", @"%d KeyPress\n",
+        NSDebugLLog(@"NSEvent", @"%lu KeyPress\n",
                     xEvent.xkey.window);
         [self setLastTime: xEvent.xkey.time];
         e = process_key_event (&xEvent, self, NSKeyDown, event_queue);
@@ -1182,7 +1183,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
 
             // a key has been released
       case KeyRelease:
-        NSDebugLLog(@"NSEvent", @"%d KeyRelease\n",
+        NSDebugLLog(@"NSEvent", @"%lu KeyRelease\n",
                     xEvent.xkey.window);
         [self setLastTime: xEvent.xkey.time];
         e = process_key_event (&xEvent, self, NSKeyUp, event_queue);
@@ -1195,9 +1196,20 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
           if (_is_keyboard_initialized == NO)
             initialize_keyboard ();
 
-          NSDebugLLog(@"NSEvent", @"%d KeymapNotify\n",
+          NSDebugLLog(@"NSEvent", @"%lu KeymapNotify\n",
                       xEvent.xkeymap.window);
 
+          // Check if shift is pressed
+          _shift_pressed = 0;
+          if (check_modifier (&xEvent, XK_Shift_L))
+            {
+              _shift_pressed |= 1;
+            }
+          if (check_modifier (&xEvent, XK_Shift_R))
+            {
+              _shift_pressed |= 2;
+            }
+
           // Check if control is pressed
           _control_pressed = 0;
           if ((_control_keysyms[0] != NoSymbol)
@@ -1255,7 +1267,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
             // when a window changes state from ummapped to
             // mapped or vice versa
       case MapNotify:
-        NSDebugLLog(@"NSEvent", @"%d MapNotify\n",
+        NSDebugLLog(@"NSEvent", @"%lu MapNotify\n",
                     xEvent.xmap.window);
         if (cWin == 0 || xEvent.xmap.window != cWin->ident)
           {
@@ -1272,7 +1284,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
             if (generic.desiredFocusWindow == cWin->number
                 && generic.focusRequestNumber == 0)
               {
-                NSDebugLLog(@"Focus", @"Refocusing %d on map notify", 
+                NSDebugLLog(@"Focus", @"Refocusing %lu on map notify", 
                             cWin->number);
                 [self setinputfocus: cWin->number];
               }
@@ -1286,7 +1298,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
 
             // Window is no longer visible.
       case UnmapNotify:
-        NSDebugLLog(@"NSEvent", @"%d UnmapNotify\n",
+        NSDebugLLog(@"NSEvent", @"%lu UnmapNotify\n",
                     xEvent.xunmap.window);
         if (cWin == 0 || xEvent.xunmap.window != cWin->ident)
           {
@@ -1302,13 +1314,13 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
 
             // like MapNotify but occurs before the request is carried out
       case MapRequest:
-        NSDebugLLog(@"NSEvent", @"%d MapRequest\n",
+        NSDebugLLog(@"NSEvent", @"%lu MapRequest\n",
                     xEvent.xmaprequest.window);
         break;
 
             // keyboard or mouse mapping has been changed by another client
       case MappingNotify:
-        NSDebugLLog(@"NSEvent", @"%d MappingNotify\n",
+        NSDebugLLog(@"NSEvent", @"%lu MappingNotify\n",
                     xEvent.xmapping.window);
         if ((xEvent.xmapping.request == MappingModifier) 
             || (xEvent.xmapping.request == MappingKeyboard))
@@ -1319,7 +1331,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
         break;
 
       case MotionNotify:
-        NSDebugLLog(@"NSMotionEvent", @"%d MotionNotify - %d %d\n",
+        NSDebugLLog(@"NSMotionEvent", @"%lu MotionNotify - %d %d\n",
                     xEvent.xmotion.window, xEvent.xmotion.x, xEvent.xmotion.y);
         {
           unsigned int        state;
@@ -1399,7 +1411,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
 
       // a window property has changed or been deleted
       case PropertyNotify:
-        NSDebugLLog(@"NSEvent", @"%d PropertyNotify - '%s'\n",
+        NSDebugLLog(@"NSEvent", @"%lu PropertyNotify - '%s'\n",
                     xEvent.xproperty.window,
                     XGetAtomName(dpy, xEvent.xproperty.atom));
 	if (xEvent.xproperty.atom == generic.wm_state_atom)
@@ -1439,7 +1451,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
 			/* Same event as when we get ClientMessage with the
 			 * atom equal to generic.miniaturize_atom
 			 */
-			NSDebugLLog(@"Miniaturize", @"%d miniaturized",
+			NSDebugLLog(@"Miniaturize", @"%lu miniaturized",
 				    cWin->number);
 			eventLocation = NSMakePoint(0,0);
 			e = [NSEvent otherEventWithType: NSAppKitDefined
@@ -1461,7 +1473,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
 		    if (cWin->wm_state == IconicState &&
 			[GSWindowWithNumber(cWin->number) isMiniaturized])
 		      {
-			NSDebugLLog(@"Miniaturize", @"%d deminiaturized",
+			NSDebugLLog(@"Miniaturize", @"%lu deminiaturized",
 				    cWin->number);
 			eventLocation = NSMakePoint(0,0);
 			e = [NSEvent otherEventWithType: NSAppKitDefined
@@ -1485,7 +1497,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
 
             // a client successfully reparents a window
       case ReparentNotify:
-        NSDebugLLog(@"NSEvent", @"%d ReparentNotify - offset %d %d\n",
+        NSDebugLLog(@"NSEvent", @"%lu ReparentNotify - offset %d %d\n",
                     xEvent.xreparent.window, xEvent.xreparent.x,
                     xEvent.xreparent.y);
         if (cWin == 0 || xEvent.xreparent.window != cWin->ident)
@@ -1551,8 +1563,8 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
                     unsigned int nchildren;
 
                     parent = new_parent;
-                    NSLog(@"QueryTree window is %d (root %d cwin root %d)", 
-                          parent, root, cWin->root);
+                    NSLog(@"QueryTree window is %lu (cwin root %lu)", 
+                          parent, cWin->root);
                     if (!XQueryTree(dpy, parent, &root, &new_parent, 
                       &children, &nchildren))
                       {
@@ -1652,23 +1664,23 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
 
             // another client attempts to change the size of a window
       case ResizeRequest:
-        NSDebugLLog(@"NSEvent", @"%d ResizeRequest\n",
+        NSDebugLLog(@"NSEvent", @"%lu ResizeRequest\n",
                     xEvent.xresizerequest.window);
         break;
 
             // events dealing with the selection
       case SelectionClear:
-        NSDebugLLog(@"NSEvent", @"%d SelectionClear\n",
+        NSDebugLLog(@"NSEvent", @"%lu SelectionClear\n",
                     xEvent.xselectionclear.window);
         break;
 
       case SelectionNotify:
-        NSDebugLLog(@"NSEvent", @"%d SelectionNotify\n",
+        NSDebugLLog(@"NSEvent", @"%lu SelectionNotify\n",
                     xEvent.xselection.requestor);
         break;
 
       case SelectionRequest:
-        NSDebugLLog(@"NSEvent", @"%d SelectionRequest\n",
+        NSDebugLLog(@"NSEvent", @"%lu SelectionRequest\n",
                     xEvent.xselectionrequest.requestor);
         {
           NSPasteboard *pb = [NSPasteboard pasteboardWithName: NSDragPboard];
@@ -1745,7 +1757,7 @@ posixFileDescriptor: (NSPosixFileDescriptor*)fileDescriptor
   NSEvent *e = nil;
   key_win = [NSApp keyWindow];
   key_num = [key_win windowNumber];
-  NSDebugLLog(@"Focus", @"take focus:%d (current=%d key=%d)",
+  NSDebugLLog(@"Focus", @"take focus:%lu (current=%lu key=%d)",
               cWin->number, generic.currentFocusWindow, key_num);
 
   /* Sometimes window managers lose the setinputfocus on the key window
@@ -1969,8 +1981,6 @@ keysym_is_X_modifier (KeySym keysym)
   switch (keysym)
     {
     case XK_Num_Lock: 
-    case XK_Shift_L:    
-    case XK_Shift_R:    
     case XK_Caps_Lock:  
     case XK_Shift_Lock: 
       return YES;
@@ -1993,6 +2003,7 @@ process_key_event (XEvent* xEvent, XGServer* context, NSEventType eventType,
   NSEvent *event = nil;
   NSEventType originalType;
   gswindow_device_t *window;
+  int shift_key = 0;
   int control_key = 0;
   int command_key = 0;
   int alt_key = 0;
@@ -2004,15 +2015,18 @@ process_key_event (XEvent* xEvent, XGServer* context, NSEventType eventType,
   if (_is_keyboard_initialized == NO)
     initialize_keyboard ();
 
-  /* Process location */
   window = [XGServer _windowWithTag: [[NSApp keyWindow] windowNumber]];
-  if (window != 0)
+  if (!window)
     {
-      eventLocation.x = xEvent->xbutton.x;
-      eventLocation.y = xEvent->xbutton.y;
-      eventLocation = [context _XPointToOSPoint: eventLocation
-                                            for: window];
+      // No key event if we don't have a key window
+      return nil;
     }
+
+  /* Process location */
+  eventLocation.x = xEvent->xbutton.x;
+  eventLocation.y = xEvent->xbutton.y;
+  eventLocation = [context _XPointToOSPoint: eventLocation
+                                        for: window];
     
   /* Process characters */
   keys = [context->inputServer lookupStringForEvent: (XKeyEvent *)xEvent
@@ -2033,7 +2047,15 @@ process_key_event (XEvent* xEvent, XGServer* context, NSEventType eventType,
       XLookupKeysym((XKeyEvent *)xEvent, 0) : keysym;
   if (modKeysym != NoSymbol)
     {
-      if (modKeysym == _control_keysyms[0]) 
+      if (modKeysym == XK_Shift_L)
+        {
+          shift_key = 1;
+        }
+      else if (modKeysym == XK_Shift_R)
+        {
+          shift_key = 2;
+        }
+      else if (modKeysym == _control_keysyms[0]) 
         {
           control_key = 1;
         }
@@ -2068,11 +2090,13 @@ process_key_event (XEvent* xEvent, XGServer* context, NSEventType eventType,
     }
 
   originalType = eventType;
-  if (control_key || command_key || alt_key || help_key)
+  if (shift_key || control_key || command_key || alt_key || help_key)
     {
       eventType = NSFlagsChanged;
       if (xEvent->xkey.type == KeyPress)
         {
+          if (shift_key)
+            _shift_pressed |= shift_key;
           if (control_key)
             _control_pressed |= control_key;
           if (command_key)
@@ -2084,6 +2108,8 @@ process_key_event (XEvent* xEvent, XGServer* context, NSEventType eventType,
         }
       else if (xEvent->xkey.type == KeyRelease)
         {
+          if (shift_key)
+            _shift_pressed &= ~shift_key;
           if (control_key)
             _control_pressed &= ~control_key;
           if (command_key)
@@ -2102,7 +2128,7 @@ process_key_event (XEvent* xEvent, XGServer* context, NSEventType eventType,
   if (IsKeypadKey (keysym))
     eventFlags = eventFlags | NSNumericPadKeyMask;
 
-  NSDebugLLog (@"NSKeyEvent", @"keysym=%d, keyCode=%d flags=%d (state=%d)",
+  NSDebugLLog (@"NSKeyEvent", @"keysym=%lu, keyCode=%d flags=%d (state=%d)",
               keysym, keyCode, eventFlags, ((XKeyEvent *)xEvent)->state);
   
   /* Add NSFunctionKeyMask if the key is a function or a misc function key */
@@ -2269,11 +2295,11 @@ process_char (KeySym keysym, unsigned *eventModifierFlags)
     case XK_Linefeed:     return NSFormFeedCharacter;
     case XK_Tab:          return NSTabCharacter;
 #ifdef XK_XKB_KEYS
-    case XK_ISO_Left_Tab: return NSTabCharacter;
+    case XK_ISO_Left_Tab: return NSBackTabCharacter;
 #endif
       /* FIXME: The following line ? */
     case XK_Escape:       return 0x1b;
-    case XK_BackSpace:    return NSBackspaceKey;
+    case XK_BackSpace:    return NSDeleteCharacter;
 
       /* The following keys need to be reported as function keys */
 #define XGPS_FUNCTIONKEY \
@@ -2374,7 +2400,7 @@ process_modifier_flags(unsigned int state)
 {
   unsigned int eventModifierFlags = 0;
 
-  if (state & ShiftMask)
+  if (_shift_pressed != 0)
     eventModifierFlags = eventModifierFlags | NSShiftKeyMask;
 
   if (state & LockMask)
