@@ -1541,7 +1541,13 @@ static Region emptyRegion;
 
   if (pattern != nil)
     {
-      [self fillRect: NSMakeRect(x, y, w, h) withPattern: pattern];
+      NSBezierPath *path;
+
+      path = [[NSBezierPath alloc] init];
+      [path appendBezierPathWithRect: NSMakeRect(x, y, w, h)];
+      [path transformUsingAffineTransform: ctm];
+      [self fillPath: path withPattern: pattern];
+      RELEASE(path);
       return;
     }
 
